/**	\file wsdl.h
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008-2009  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _Wsdl_H
#define _Wsdl_H

#include <string>
#include <list>
#include <Sockets/XmlNode.h>


class Application;

class Wsdl
{
	class NSFil {
	public:
		NSFil(FILE *fil, const std::string& ns, const std::string& href = "") : m_fil(fil), m_ns(ns), m_href(href), m_start(false) {}
		void Begin() {
			if (!m_start)
			{
				if (!m_ns.empty())
				{
					if (!m_href.empty())
						fprintf(m_fil, "/** %s */\n", m_href.c_str());
					fprintf(m_fil, "namespace %s {\n\n", m_ns.c_str());
				}
				m_start = true;
			}
		}
		~NSFil() {
			if (m_start)
			{
				if (!m_ns.empty())
				{
					fprintf(m_fil, "} // namespace %s\n\n", m_ns.c_str());
				}
			}
		}
	private:
		FILE *m_fil;
		std::string m_ns;
		std::string m_href;
		bool m_start;
	};

public:
	Wsdl(Application& appl, XmlDocument& doc);
	~Wsdl();

	Application& GetApplication() const { return m_appl; }

	const std::string GetTargetNamespace() const;
	const std::string GetTargetNamespacePrefix() const;

	std::list<std::string> GetPorts() const;
	const std::string GetPortname() const;
	std::list<std::string> GetOperations(const std::string& portname) const;

	const std::string GetSoapAction(const std::string& portname, const std::string& op) const;

	const std::string GetMessageForOp(const std::string& portname, const std::string& op, const std::string& type) const;

	void ShowMessage(const std::string& msgname);
	void ShowElement(const XmlNode& element_node, const std::string& level);
	void ShowComplexType(const XmlNode& complex, const std::string& level);

	std::list<std::pair<std::string, XmlNode> > GetMessageParameters(const std::string& msgname, std::string& path) const;

	XmlNode GetPortTypeOp(const std::string& portname, const std::string& op) const;
	XmlNode GetBindingOp(const std::string& portname, const std::string& op) const;
	XmlNode GetMessage(const std::string& msgname) const;
	XmlNode GetSchema(const std::string& target_ns) const;
	XmlNode GetElement(const std::string& name) const;
	XmlNode FindComplexType(const std::string& ns_href, const std::string& type) const;
	XmlNode FindSimpleType(const std::string& ns_href, const std::string& type) const;
	XmlNode GetElement(const XmlNode& node, const std::string& name) const;

	/** types */
	void CreateInterface();
	void CreateImplementation();

	const std::string GetParent(const std::string& href, const std::string& classname) const;

	const std::list<std::string>& ComplexTypes(const std::string& ns_href) const;
	const std::list<std::string>& SimpleTypes(const std::string& ns_href) const;

	const std::string NamespaceFromHref(const std::string& href) const;

	/** Return prefix or mangled href */
	const std::string GetNamespace(const std::string& ns_href) const;

private:
	void ParseElements(XmlDocument& doc, const std::string& targetNs = "");
	void ParseElements(const XmlNode& node, const std::string& ns_href, const std::string& parent_class = "");
	void ParseTypes(const XmlNode& node, const std::string& ns_href);
	XmlNode ParseComplexType(const XmlNode& node, const std::string& ns_href, const std::string& name);
	void AddLicense(FILE *);

	Application& m_appl;

	/** Main document */
	XmlDocument& m_doc;

	/** Imported documents */
	std::list<XmlDocument *> m_import;

	/** List of complexType names in each namespace */
	std::map<std::string, std::list<std::string> > m_complexType;
	/** List of simpleType names in each namespace */
	std::map<std::string, std::list<std::string> > m_simpleType;
	/** Map complexType names in each namespace to XmlNode of definition */
	std::map<std::string, std::map<std::string, XmlNode> > m_complexTypeMap;
	/** Map simpleType names in each namespace to XmlNode of definition */
	std::map<std::string, std::map<std::string, XmlNode> > m_simpleTypeMap;

	/** parent class map */
	std::map<std::string, std::map<std::string, std::string> > m_parent; // <ns_href, <classname, parent classname> >

};




#endif // _Wsdl_H
